
#adapted from layers_materials.rb from Didier Bur's website:
require 'sketchup.rb'

	def convert_layer_names_to_materials
		prompts = ["Create new materials for all layers (except Layer0)?", "Do not change existing, non default, material names?"]
		defaults = ["Yes", "Yes",]
		list = ["Yes|No", "Yes|No"]
		results = UI.inputbox(prompts, defaults, list, "Convert Layer Names to Materials")
		return if !results
		bcomm_make_dont_override_materials = false
		if (results[1] == "Yes")
			bcomm_make_dont_override_materials = true
		end#if
		convert = CL_convert_layer_materials.new(bcomm_make_dont_override_materials)
		
		if (results[0] == "Yes")			
			convert.create_materials_from_layers
		end#if
		convert.convert_layer_names_to_materials
	end#def
	

	class CL_convert_layer_materials 
		attr_accessor :count
		attr_accessor :matched
		attr_accessor :already_set
		attr_accessor :ignored
		attr_accessor :component_names
		attr_accessor :materials_to_process
		attr_accessor :bcomm_make_dont_override_materials
		
		def initialize(bcomm_make_dont_override_materials)
			@count = 0
			@mateched = 0
			@already_set = 0
			@ignored = 0
			@component_names = {} # hash of component names to proceess
			@materials_to_process = {}
			@bcomm_make_dont_override_materials = bcomm_make_dont_override_materials
		end#def

	
		def convert_layer_names_to_materials
			model = Sketchup.active_model
			layers = model.layers
			ents = model.entities
			
			# get all material available in model
			materials = model.materials
			@materials_to_process = {}
			materials.each do |m|
			  materials_to_process[m.display_name] = 0
			end#loop
			$materials_to_process = materials_to_process


			@matched = 0
			@ignored = 0
			@already_set = 0
			
			model.start_operation "Convert Materials to Layer Names"
			process_entities(model.active_entities, 0)
			model.commit_operation
			
			smess = sprintf("Parsed: %s faced - changed: %s skipped: %s already_set: %s",
				matched+ignored+already_set, matched, ignored, already_set)
			UI.messagebox(smess)
			
		end#def
		
		def process_entities(ents, level)
			printf("Process: %s %s\n", ents.length, level)

			
			ents.each do |entity|
			  type = entity.typename
			  case type
				when "Face"
				  if materials_to_process[entity.layer.name]
					materials_to_process[entity.layer.name] = 1
					if (!bcomm_make_dont_override_materials || (!entity.material && !entity.back_material))
						entity.material = entity.layer.name
						entity.back_material = entity.layer.name
						@matched = matched + 1
					else
						@already_set = already_set + 1
					end#if
				  else
					@ignored = ignored + 1
				  end#if
				when "Group"
					# don't process if group has a material and not overriding
					if (!bcomm_make_dont_override_materials || !entity.material)
						printf("Process Group: %s\n", entity)
						process_entities(entity.entities, level + 1)
					end#if
				when "ComponentInstance"
					# don't process if group has a material and not overriding
					if (!bcomm_make_dont_override_materials || !entity.material)	
						# Only process each component definition once					
						if (!component_names[entity.definition.name])
							printf("Process Component Definition: %s\n", entity.definition.name)
							component_names[entity.definition.name] = 1
							process_entities(entity.definition.entities, level + 1)
						end#if
					end#if
				when "Edge"
					# skip edges
				when "Image"
					# skip edges
				else
					printf("Unknown entity: %s\n", entity)
					$entity = entity
			  end#case
			end#loop
		end#def
		
	
		def create_materials_from_layers
			model = Sketchup.active_model
			materials = model.materials
			mats = []
			materials.each do |m|
			  mats.push m.name
			end
			new = 0
			model.start_operation "Create materials from layers"
			
			#skip Layer0
			model.layers.to_a[1..-1].each { |layer|
			  if not mats.include? layer.name
				#This is a new layer without corresponding material so create one
				printf("Make new Material: %s\n", layer.name)
				new_mat = Sketchup.active_model.materials.add layer.name
				new_mat.color = layer.color
				new = new + 1
			  end#if
			}
			UI.messagebox(new.to_s + " new materials created.")
			model.commit_operation
		end#def
	
	end#class
	
if not file_loaded?("convert_materials_to_layer_names.rb")
	 UI.menu("Plugins").add_item("Convert Layer Names+Colors to Materials") { convert_layer_names_to_materials }
end#if
#-----------------------------------------------------------------------------
file_loaded("convert_materials_to_layer_names.rb")

